@@ -39,10 +39,10 @@
  *
  * Last reviewed on 2006-03-01 (0.10.4)
  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -51,7 +51,8 @@
 #include <gst/gsttagsetter.h>
 #include <gst/tag/tag.h>
 #include <gst/audio/multichannel.h>
-#include "vorbisenc.h"
+#include <gst/audio/audio.h>
+#include "gstvorbisenc.h"
 
 GST_DEBUG_CATEGORY_EXTERN (vorbisenc_debug);
 #define GST_CAT_DEFAULT vorbisenc_debug
@@ -144,9 +145,11 @@ static void
 gst_vorbis_enc_add_interfaces (GType vorbisenc_type)
 {
   static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
+  static const GInterfaceInfo preset_info = { NULL, NULL, NULL };
 
   g_type_add_interface_static (vorbisenc_type, GST_TYPE_TAG_SETTER,
       &tag_setter_info);
+  g_type_add_interface_static (vorbisenc_type, GST_TYPE_PRESET, &preset_info);
 }
 
 static void
@@ -893,7 +896,8 @@ gst_vorbis_enc_buffer_from_packet (GstVorbisEnc * vorbisenc,
   /* update the next timestamp, taking granulepos_offset and subgranule offset
    * into account */
   vorbisenc->next_ts =
-      granulepos_to_timestamp_offset (vorbisenc, packet->granulepos);
+      granulepos_to_timestamp_offset (vorbisenc, packet->granulepos) +
+      vorbisenc->initial_ts;
   GST_BUFFER_DURATION (outbuf) =
       vorbisenc->next_ts - GST_BUFFER_TIMESTAMP (outbuf);
 
@@ -902,6 +906,8 @@ gst_vorbis_enc_buffer_from_packet (GstVorbisEnc * vorbisenc,
     vorbisenc->next_discont = FALSE;
   }
 
+  gst_buffer_set_caps (outbuf, vorbisenc->srccaps);
+
   GST_LOG_OBJECT (vorbisenc, "encoded buffer of %d bytes",
       GST_BUFFER_SIZE (outbuf));
   return outbuf;
@@ -1034,9 +1040,15 @@ gst_vorbis_enc_sink_event (GstPad * pad, GstEvent * event)
 
       gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
           &format, &start, &stop, &position);
-      if (format == GST_FORMAT_TIME)
+      if (format == GST_FORMAT_TIME) {
         gst_segment_set_newsegment (&vorbisenc->segment, update, rate, format,
             start, stop, position);
+        if (vorbisenc->initial_ts == GST_CLOCK_TIME_NONE) {
+          GST_DEBUG_OBJECT (vorbisenc, "Initial segment %" GST_SEGMENT_FORMAT,
+              &vorbisenc->segment);
+          vorbisenc->initial_ts = start;
+        }
+      }
     }
       /* fall through */
     default:
@@ -1089,16 +1101,25 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
   GstBuffer *buf1, *buf2, *buf3;
   gboolean first = FALSE;
   GstClockTime timestamp = GST_CLOCK_TIME_NONE;
+  GstClockTime running_time = GST_CLOCK_TIME_NONE;
 
   vorbisenc = GST_VORBISENC (GST_PAD_PARENT (pad));
 
   if (!vorbisenc->setup)
     goto not_setup;
 
-  timestamp =
+  buffer = gst_audio_buffer_clip (buffer, &vorbisenc->segment,
+      vorbisenc->frequency, 4 * vorbisenc->channels);
+  if (buffer == NULL) {
+    GST_DEBUG_OBJECT (vorbisenc, "Dropping buffer, out of segment");
+    return GST_FLOW_OK;
+  }
+  running_time =
       gst_segment_to_running_time (&vorbisenc->segment, GST_FORMAT_TIME,
       GST_BUFFER_TIMESTAMP (buffer));
-
+  timestamp = running_time + vorbisenc->initial_ts;
+  GST_DEBUG_OBJECT (vorbisenc, "Initial ts is %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (vorbisenc->initial_ts));
   if (!vorbisenc->header_sent) {
     /* Vorbis streams begin with three headers; the initial header (with
        most of the codec setup parameters) which is mandated by the Ogg
@@ -1158,10 +1179,11 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
     vorbisenc->next_ts = timestamp;
     vorbisenc->expected_ts = timestamp;
     vorbisenc->granulepos_offset = gst_util_uint64_scale
-        (timestamp, vorbisenc->frequency, GST_SECOND);
+        (running_time, vorbisenc->frequency, GST_SECOND);
     vorbisenc->subgranule_offset = 0;
     vorbisenc->subgranule_offset =
-        vorbisenc->next_ts - granulepos_to_timestamp_offset (vorbisenc, 0);
+        (vorbisenc->next_ts - vorbisenc->initial_ts) -
+        granulepos_to_timestamp_offset (vorbisenc, 0);
 
     vorbisenc->header_sent = TRUE;
     first = TRUE;
@@ -1211,7 +1233,7 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
     /* We need to round to the nearest whole number of samples, not just do
      * a truncating division here */
     vorbisenc->granulepos_offset = gst_util_uint64_scale
-        (timestamp + GST_SECOND / vorbisenc->frequency / 2
+        (running_time + GST_SECOND / vorbisenc->frequency / 2
         - vorbisenc->subgranule_offset, vorbisenc->frequency, GST_SECOND);
 
     vorbisenc->header_sent = TRUE;
@@ -1426,6 +1448,7 @@ gst_vorbis_enc_change_state (GstElement * element, GstStateChange transition)
       vorbisenc->next_discont = FALSE;
       vorbisenc->header_sent = FALSE;
       gst_segment_init (&vorbisenc->segment, GST_FORMAT_TIME);
+      vorbisenc->initial_ts = GST_CLOCK_TIME_NONE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
