@@ -41,6 +41,7 @@ enum
   ARG_0,
   ARG_LABEL,
   ARG_UNTRANSLATED_LABEL,
+  ARG_INDEX,
   ARG_MIN_VOLUME,
   ARG_MAX_VOLUME,
   ARG_FLAGS,
@@ -117,6 +118,20 @@ gst_mixer_track_class_init (GstMixerTrackClass * klass)
           NULL,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstMixerTrack:index
+   *
+   * Index of the mixer track, if available. Mixer track implementations
+   * must set this at construct time. This can be used to discern between
+   * multiple tracks with identical labels.
+   *
+   * Since: 0.10.21
+   */
+  g_object_class_install_property (object_klass, ARG_INDEX,
+      g_param_spec_uint ("index", "Index",
+          "Track index", 0, G_MAXUINT,
+          0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
   g_object_class_install_property (object_klass, ARG_MIN_VOLUME,
       g_param_spec_int ("min-volume", "Minimum volume level",
           "The minimum possible volume level", G_MININT, G_MAXINT,
@@ -173,8 +188,9 @@ gst_mixer_track_init (GstMixerTrack * mixer_track)
   mixer_track->num_channels = 0;
 }
 
-/* FIXME 0.11: move this as a member into the mixer track structure */
+/* FIXME 0.11: move these as members into the mixer track structure */
 #define MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL "gst-mixer-track-ulabel"
+#define MIXER_TRACK_OBJECT_DATA_KEY_INDEX "index"
 
 static void
 gst_mixer_track_get_property (GObject * object, guint prop_id, GValue * value,
@@ -193,6 +209,11 @@ gst_mixer_track_get_property (GObject * object, guint prop_id, GValue * value,
           (const gchar *) g_object_get_data (G_OBJECT (mixer_track),
               MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL));
       break;
+    case ARG_INDEX:
+      g_value_set_uint (value,
+          GPOINTER_TO_INT (g_object_get_data (G_OBJECT (mixer_track),
+                  MIXER_TRACK_OBJECT_DATA_KEY_INDEX)));
+      break;
     case ARG_MIN_VOLUME:
       g_value_set_int (value, mixer_track->min_volume);
       break;
@@ -225,6 +246,11 @@ gst_mixer_track_set_property (GObject * object, guint prop_id,
           MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL,
           g_value_dup_string (value), (GDestroyNotify) g_free);
       break;
+    case ARG_INDEX:
+      g_object_set_data (G_OBJECT (mixer_track),
+          MIXER_TRACK_OBJECT_DATA_KEY_INDEX,
+          GINT_TO_POINTER (g_value_get_uint (value)));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
