@@ -41,10 +41,34 @@ typedef struct _GstBaseRTPPayload GstBaseRTPPayload;
 typedef struct _GstBaseRTPPayloadPrivate GstBaseRTPPayloadPrivate;
 typedef struct _GstBaseRTPPayloadClass GstBaseRTPPayloadClass;
 
+/**
+ * GST_BASE_RTP_PAYLOAD_SINKPAD:
+ * @payload: a #GstBaseRTPPayload
+ *
+ * Get access to the sinkpad of @payload.
+ */
 #define GST_BASE_RTP_PAYLOAD_SINKPAD(payload) (GST_BASE_RTP_PAYLOAD (payload)->sinkpad)
+/**
+ * GST_BASE_RTP_PAYLOAD_SRCPAD:
+ * @payload: a #GstBaseRTPPayload
+ *
+ * Get access to the srcpad of @payload.
+ */
 #define GST_BASE_RTP_PAYLOAD_SRCPAD(payload)  (GST_BASE_RTP_PAYLOAD (payload)->srcpad)
 
+/**
+ * GST_BASE_RTP_PAYLOAD_PT:
+ * @payload: a #GstBaseRTPPayload
+ *
+ * Get access to the configured payload type of @payload.
+ */
 #define GST_BASE_RTP_PAYLOAD_PT(payload)  (GST_BASE_RTP_PAYLOAD (payload)->pt)
+/**
+ * GST_BASE_RTP_PAYLOAD_MTU:
+ * @payload: a #GstBaseRTPPayload
+ *
+ * Get access to the configured MTU of @payload.
+ */
 #define GST_BASE_RTP_PAYLOAD_MTU(payload) (GST_BASE_RTP_PAYLOAD (payload)->mtu)
 
 struct _GstBaseRTPPayload
@@ -94,13 +118,14 @@ struct _GstBaseRTPPayloadClass
   /* receive caps on the sink pad, configure the payloader. */
   gboolean      (*set_caps)             (GstBaseRTPPayload *payload, GstCaps *caps);
   /* handle a buffer, perform 0 or more gst_basertppayload_push() on
-   * the RTP buffers */
+   * the RTP buffers. This function takes ownership of the buffer. */
   GstFlowReturn (*handle_buffer)        (GstBaseRTPPayload *payload, 
                                          GstBuffer *buffer);
   gboolean      (*handle_event)         (GstPad * pad, GstEvent * event);
+  GstCaps *     (*get_caps)             (GstBaseRTPPayload *payload, GstPad * pad);
 
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING-1];
+  gpointer _gst_reserved[GST_PADDING-2];
 };
 
 GType           gst_basertppayload_get_type             (void);
@@ -119,6 +144,9 @@ gboolean        gst_basertppayload_is_filled            (GstBaseRTPPayload *payl
 GstFlowReturn   gst_basertppayload_push                 (GstBaseRTPPayload *payload, 
                                                          GstBuffer *buffer);
 
+GstFlowReturn   gst_basertppayload_push_list            (GstBaseRTPPayload *payload,
+                                                         GstBufferList *list);
+
 G_END_DECLS
 
 #endif /* __GST_BASE_RTP_PAYLOAD_H__ */
