@@ -249,18 +249,28 @@ static const GstTagEntryMatch user_tag_matches[] = {
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|musicbrainz_albumid"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|musicbrainz_albumartistid"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|musicbrainz_trmid"},
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|musicbrainz_discid"},
   /* musicbrainz identifiers according to spec no one pays
    * attention to (http://musicbrainz.org/docs/specs/metadata_tags.html) */
   {GST_TAG_MUSICBRAINZ_ARTISTID, "TXXX|MusicBrainz Artist Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|MusicBrainz Album Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|MusicBrainz Album Artist Id"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|MusicBrainz TRM Id"},
+  /* according to: http://wiki.musicbrainz.org/MusicBrainzTag (yes, no space
+   * before 'ID' and not 'Id' either this time, yay for consistency) */
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|MusicBrainz DiscID"},
   /* foobar2000 uses these identifiers to store gain/peak information in
    * ID3v2 tags <= v2.3.0. In v2.4.0 there's the RVA2 frame for that */
   {GST_TAG_TRACK_GAIN, "TXXX|replaygain_track_gain"},
   {GST_TAG_TRACK_PEAK, "TXXX|replaygain_track_peak"},
   {GST_TAG_ALBUM_GAIN, "TXXX|replaygain_album_gain"},
-  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"}
+  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"},
+  /* the following two are more or less made up, there seems to be little
+   * evidence that any popular application is actually putting this info
+   * into TXXX frames; the first one comes from a musicbrainz wiki 'proposed
+   * tags' page, the second one is analogue to the vorbis/ape/flac tag. */
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|discid"},
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|CDDB DiscID"}
 };
 
 /**
@@ -283,7 +293,8 @@ gst_tag_from_id3_user_tag (const gchar * type, const gchar * id3_user_tag)
 
   for (i = 0; i < G_N_ELEMENTS (user_tag_matches); ++i) {
     if (strncmp (type, user_tag_matches[i].original_tag, 4) == 0 &&
-        strcmp (id3_user_tag, user_tag_matches[i].original_tag + 5) == 0) {
+        g_ascii_strcasecmp (id3_user_tag,
+            user_tag_matches[i].original_tag + 5) == 0) {
       GST_LOG ("Mapped ID3v2 user tag '%s' to GStreamer tag '%s'",
           user_tag_matches[i].original_tag, user_tag_matches[i].gstreamer_tag);
       return user_tag_matches[i].gstreamer_tag;
@@ -371,7 +382,7 @@ gst_tag_list_new_from_id3v1 (const guint8 * data)
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_DATE, date, NULL);
     g_date_free (date);
   }
-  if (data[125] == 0) {
+  if (data[125] == 0 && data[126] != 0) {
     gst_tag_extract_id3v1_string (list, GST_TAG_COMMENT, (gchar *) & data[97],
         28);
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_TRACK_NUMBER,
@@ -380,7 +391,7 @@ gst_tag_list_new_from_id3v1 (const guint8 * data)
     gst_tag_extract_id3v1_string (list, GST_TAG_COMMENT, (gchar *) & data[97],
         30);
   }
-  if (data[127] < gst_tag_id3_genre_count ()) {
+  if (data[127] < gst_tag_id3_genre_count () && !gst_tag_list_is_empty (list)) {
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_GENRE,
         gst_tag_id3_genre_get (data[127]), NULL);
   }
@@ -417,3 +428,58 @@ gst_tag_id3_genre_get (const guint id)
     return NULL;
   return genres[id];
 }
+
+/**
+ * gst_tag_list_add_id3_image:
+ * @tag_list: a tag list
+ * @image_data: the (encoded) image
+ * @image_data_len: the length of the encoded image data at @image_data
+ * @id3_picture_type: picture type as per the ID3 (v2.4.0) specification for
+ *    the APIC frame (0 = unknown/other)
+ *
+ * Adds an image from an ID3 APIC frame (or similar, such as used in FLAC)
+ * to the given tag list. Also see gst_tag_image_data_to_image_buffer() for
+ * more information on image tags in GStreamer.
+ *
+ * Returns: %TRUE if the image was processed, otherwise %FALSE
+ *
+ * Since: 0.10.20
+ */
+gboolean
+gst_tag_list_add_id3_image (GstTagList * tag_list, const guint8 * image_data,
+    guint image_data_len, guint id3_picture_type)
+{
+  GstTagImageType tag_image_type;
+  const gchar *tag_name;
+  GstBuffer *image;
+
+  g_return_val_if_fail (GST_IS_TAG_LIST (tag_list), FALSE);
+  g_return_val_if_fail (image_data != NULL, FALSE);
+  g_return_val_if_fail (image_data_len > 0, FALSE);
+
+  if (id3_picture_type == 0x01 || id3_picture_type == 0x02) {
+    /* file icon for preview. Don't add image-type to caps, since there
+     * is only supposed to be one of these, and the type is already indicated
+     * via the special tag */
+    tag_name = GST_TAG_PREVIEW_IMAGE;
+    tag_image_type = GST_TAG_IMAGE_TYPE_NONE;
+  } else {
+    tag_name = GST_TAG_IMAGE;
+
+    /* Remap the ID3v2 APIC type our ImageType enum */
+    if (id3_picture_type >= 0x3 && id3_picture_type <= 0x14)
+      tag_image_type = (GstTagImageType) (id3_picture_type - 2);
+    else
+      tag_image_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
+  }
+
+  image = gst_tag_image_data_to_image_buffer (image_data, image_data_len,
+      tag_image_type);
+
+  if (image == NULL)
+    return FALSE;
+
+  gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, image, NULL);
+  gst_buffer_unref (image);
+  return TRUE;
+}
